@@ -11,25 +11,15 @@ extern "C" {
 #include <unicode/translit.h>
 #include <sys/time.h>
 
-#define XAPIAN_FILE_PREFIX "xapian-indexes"
-#define XAPIAN_TERM_SIZELIMIT 245L
-#define XAPIAN_COMMIT_ENTRIES 1000000L
-#define XAPIAN_COMMIT_TIMEOUT 300L
-#define XAPIAN_WILDCARD "wldcrd"
-#define XAPIAN_EXPUNGE_HEADER 9
-#define XAPIAN_MIN_RAM 200L
-
 #define HDRS_NB 11
 static const char * hdrs_emails[HDRS_NB] = { "uid", "subject", "from", "to",  "cc",  "bcc",  "messageid", "listid", "body", "expungeheader",	""  };
 static const char * hdrs_xapian[HDRS_NB] = { "Q",   "S",       "A",    "XTO", "XCC", "XBCC", "XMID",      "XLIST",  "XBDY", "XEXP",		"XBDY" };
-
-static int verbose = 0;
+#define XAPIAN_EXPUNGE_HEADER 9
 
 struct xapian_fts_backend
 {
 	struct fts_backend backend;
 	char * path = NULL;
-	long partial,full;
 
 	char * guid;
 	char * boxname;
@@ -47,9 +37,6 @@ struct xapian_fts_backend
 	long perf_nb;
 	long perf_uid;
 	long perf_dt;
-
-	long nb_pushes;
-	long max_push;
 };
 
 struct xapian_fts_backend_update_context
@@ -61,6 +48,8 @@ struct xapian_fts_backend_update_context
 	uint32_t tbi_uid=0;
 };
 
+static struct fts_xapian_settings fts_xapian_settings;
+
 #include "fts-backend-xapian-functions.cpp"
 
 
@@ -77,11 +66,6 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 {
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)_backend;
 
-	if(verbose>0) i_info("fts_backend_xapian_init : %s",_backend->name);
-
-	const char *const *tmp, *env;
-	long len;
-
 	backend->db = NULL;
 	backend->dbw = NULL;
 
@@ -89,72 +73,13 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 	backend->path = NULL;
 	backend->old_guid = NULL;
 	backend->old_boxname = NULL;
-	verbose = 0;
-	backend->partial = 0;
-	backend->full = 0;
 
-	backend->nb_pushes=0;
-	backend->max_push=0;
-
-	env = mail_user_plugin_getenv(_backend->ns->user, "fts_xapian");
-	if (env == NULL)
-	{
-		i_error("FTS Xapian: missing configuration");
-		return -1;
-	}
-
-	for (tmp = t_strsplit_spaces(env, " "); *tmp != NULL; tmp++)
-	{
-		if (strncmp(*tmp, "partial=",8)==0)
-		{
-			len=atol(*tmp + 8);
-			if(len>0) backend->partial=len;
-		}
-		else if (strncmp(*tmp,"full=",5)==0)
-		{
-			len=atol(*tmp + 5);
-			if(len>0) backend->full=len;
-		}
-		else if (strncmp(*tmp,"verbose=",8)==0)
-		{
-			len=atol(*tmp + 8);
-			if(len>0) verbose=len;
-		}
-		else if (strncmp(*tmp,"attachments=",12)==0)
-		{
-			// Legacy
-		}
-		else
-		{
-			i_error("FTS Xapian: Invalid setting: %s", *tmp);
-			return -1;
-		}
-	}
-
-	if(backend->partial < 2)
-	{
-		i_error("FTS Xapian: 'partial' parameter is incorrect (%ld). Try 'partial=2'",backend->partial);
-		return -1;
-	}
-	if(backend->full<1)
-	{
-		i_error("FTS Xapian: 'full' parameter is incorrect (%ld). Try 'full=20'",backend->full);
-		return -1;
-	}
-	if(backend->partial > backend->full)
-	{
-		i_error("FTS Xapian: 'full' parameter must be equal or greater than 'partial'");
-		return -1;
-	}
-	if(backend->full > 50)
-	{
-		i_error("FTS Xapian: 'full' parameter above 50 is not realistic");
-		return -1;
-	}
+	struct fts_xapian_user *fuser = FTS_XAPIAN_USER_CONTEXT(_backend->ns->user);
+	fts_xapian_settings = fuser->set;
 
 	if(fts_backend_xapian_set_path(backend)<0) return -1;
 
-	if(verbose>0) i_info("FTS Xapian: Starting with partial=%ld full=%ld verbose=%d",backend->partial,backend->full,verbose);
+        if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Starting with partial=%ld full=%ld verbose=%d lowmemory=%ld",fts_xapian_settings.partial,fts_xapian_settings.full,fts_xapian_settings.verbose,fts_xapian_settings.lowmemory);
 
 	return 0;
 }
@@ -163,7 +88,7 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 {
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)_backend;
 
-	if(verbose>0) i_info("FTS Xapian: Deinit %s)",backend->path);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Deinit %s)",backend->path);
 
 	if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
 
@@ -182,7 +107,7 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 
 static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct mailbox *box, uint32_t *last_uid_r)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_get_last_uid");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_get_last_uid");
 
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)_backend;
 
@@ -198,7 +123,7 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 
 	if(!fts_backend_xapian_open_readonly(backend, &dbr))
 	{
-		if(verbose>0) i_info("FTS Xapian: GetLastUID: Can not open db RO (%s)",backend->db);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: GetLastUID: Can not open db RO (%s)",backend->db);
 		return 0;
 	}
 
@@ -218,7 +143,7 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 	dbr->close();
 	delete(dbr);
 
-	if(verbose>0) i_info("FTS Xapian: Get last UID of %s (%s) = %d",backend->boxname,backend->guid,*last_uid_r);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Get last UID of %s (%s) = %d",backend->boxname,backend->guid,*last_uid_r);
 
 	return 0;
 }
@@ -226,7 +151,7 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 
 static struct fts_backend_update_context * fts_backend_xapian_update_init(struct fts_backend *_backend)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_update_context");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_update_context");
 
 	struct xapian_fts_backend_update_context *ctx;
 
@@ -240,7 +165,7 @@ static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)ctx->ctx.backend;
 
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_deinit (%s)",backend->path);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_deinit (%s)",backend->path);
 
 	fts_backend_xapian_release(backend,"update_deinit",0);
 
@@ -251,7 +176,7 @@ static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_
 
 static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_context *_ctx, struct mailbox *box)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_set_mailbox");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_set_mailbox");
 
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)ctx->ctx.backend;
@@ -261,7 +186,7 @@ static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_cont
 
 static void fts_backend_xapian_update_expunge(struct fts_backend_update_context *_ctx, uint32_t uid)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_expunge");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_expunge");
 
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *)ctx->ctx.backend;
@@ -274,7 +199,7 @@ static void fts_backend_xapian_update_expunge(struct fts_backend_update_context
 
 	try
 	{
-		if(verbose>0) i_info("FTS Xapian: Flagging UID=%d for expunge",uid);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Flagging UID=%d for expunge",uid);
 
 		XQuerySet * xq = new XQuerySet();
 		char *u = i_strdup_printf("%d",uid);
@@ -309,13 +234,13 @@ static void fts_backend_xapian_update_expunge(struct fts_backend_update_context
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: Expunging UID=%d %s",uid,e.get_msg().c_str());
+		i_error("FTS Xapian: Expunging (6) UID=%d %s",uid,e.get_msg().c_str());
 	}
 }
 
 static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_context *_ctx, const struct fts_backend_build_key *key)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_update_set_build_key");
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_update_set_build_key");
 
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 
@@ -326,7 +251,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 
 	if(backend->guid == NULL)
 	{
-		if(verbose>0) i_warning("FTS Xapian: Build key %s with no mailbox",key->hdr_name);
+		if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: Build key %s with no mailbox",key->hdr_name);
 		return FALSE;
 	}
 
@@ -335,7 +260,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 		fts_backend_xapian_oldbox(backend);
 		backend->old_guid = i_strdup(backend->guid);
 		backend->old_boxname = i_strdup(backend->boxname);
-		if(verbose>0) i_info("FTS Xapian: Start indexing '%s' (%s)",backend->boxname,backend->guid);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Start indexing '%s' (%s)",backend->boxname,backend->guid);
 	}
 
 	/* Performance calculator*/
@@ -354,7 +279,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 			r=backend->perf_nb*1000.0;
 			r=r/dt;
 		}
-		if(verbose>0) i_info("FTS Xapian: Partial indexing '%s' (%ld msgs in %ld ms, rate: %.1f)",backend->boxname,backend->perf_nb,dt,r);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Partial indexing '%s' (%ld msgs in %ld ms, rate: %.1f)",backend->boxname,backend->perf_nb,dt,r);
 	}
 	/* End Performance calculator*/
 
@@ -362,19 +287,19 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 	const char * type = key->body_content_type;
 	const char * disposition = key->body_content_disposition;
 
-	if(verbose>1) i_info("FTS Xapian: New part (Header=%s,Type=%s,Disposition=%s)",field,type,disposition);
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: New part (Header=%s,Type=%s,Disposition=%s)",field,type,disposition);
 
 	// Verify content-type
 
 	if(key->type == FTS_BACKEND_BUILD_KEY_BODY_PART_BINARY)
 	{
-		if(verbose>0) i_info("FTS Xapian: Skipping binary part of type '%s'",type);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Skipping binary part of type '%s'",type);
 		return FALSE;
 	}
 
 	if((type != NULL) && (strncmp(type,"text",4)!=0) && ((disposition==NULL) || ((strstr(disposition,"filename=")==NULL) && (strstr(disposition,"attachment")==NULL))))
 	{
-		if(verbose>0) i_info("FTS Xapian: Non-binary & non-text part of type '%s'",type);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Non-binary & non-text part of type '%s'",type);
 		return FALSE;
 	}
 
@@ -382,7 +307,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 	ctx->isattachment=false;
 	if((disposition != NULL) && ((strstr(disposition,"filename=")!=NULL) || (strstr(disposition,"attachment")!=NULL)))
 	{
-		if(verbose>0) i_info("FTS Xapian: Found part as attachment of type '%s' and disposition '%s'",type,disposition);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Found part as attachment of type '%s' and disposition '%s'",type,disposition);
 		ctx->isattachment=true;
 	}
 
@@ -411,7 +336,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 	}
 	if(i>=HDRS_NB)
 	{
-		if(verbose>1) i_info("FTS Xapian: Unknown header '%s' of part",ctx->tbi_field);
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: Unknown header '%s' of part",ctx->tbi_field);
 		i_free(ctx->tbi_field);
 		ctx->tbi_field=NULL;
 		return FALSE;
@@ -438,7 +363,7 @@ static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_
 {
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 
-	if(verbose>0) 
+	if(fts_xapian_settings.verbose>0) 
 	{
 		long n = 0;
 		if( ctx->ctx.backend != NULL )
@@ -467,7 +392,7 @@ static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_
 
 static int fts_backend_xapian_refresh(struct fts_backend * _backend)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_refresh");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_refresh");
 
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *) _backend;
 
@@ -481,7 +406,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 	struct xapian_fts_backend_update_context *ctx = (struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *) ctx->ctx.backend;
 
-	if(verbose>1)
+	if(fts_xapian_settings.verbose>1)
 	{
 		if(ctx->isattachment)
 		{
@@ -502,7 +427,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
 	icu::StringPiece sp_d((const char *)data,(int32_t )size);
 	icu::UnicodeString d2 = icu::UnicodeString::fromUTF8(sp_d);
-	if(d2.length() < backend->partial) return 0;
+	if(d2.length() < fts_xapian_settings.partial) return 0;
 
 	if(!fts_backend_xapian_check_access(backend))
 	{
@@ -510,9 +435,9 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		return -1;
 	}
 
-	if(!fts_backend_xapian_test_memory(backend,d2.length()))
+	if(!fts_backend_xapian_test_memory())
 	{
-		if(verbose>0) i_info("FTS Xapian: Warning Low memory");
+		if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: Warning Low memory (%ld MB)",long(fts_backend_xapian_get_free_memory()/1024.0));
 		fts_backend_xapian_release(backend,"Low memory indexing", 0);
 		if(!fts_backend_xapian_check_access(backend))
 		{
@@ -528,7 +453,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		ok=fts_backend_xapian_index_hdr(backend,ctx->tbi_uid,ctx->tbi_field, &d2);
 		if(!ok)
 		{
-			if(verbose>0) i_info("FTS Xapian: Flushing memory and retrying");
+			if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Flushing memory and retrying");
 			fts_backend_xapian_release(backend,"Flushing memory indexing hdr", 0);
                 	if(fts_backend_xapian_check_access(backend))
 			{
@@ -545,7 +470,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		ok=fts_backend_xapian_index_text(backend,ctx->tbi_uid,ctx->tbi_field, &d2);
 		if(!ok)
 		{
-			if(verbose>0) i_info("FTS Xapian: Flushing memory and retrying");
+			if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Flushing memory and retrying");
 			fts_backend_xapian_release(backend,"Flushing memory indexing text", 0);
 			if(fts_backend_xapian_check_access(backend))
 			{
@@ -564,7 +489,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
 	if( (!ok) || (backend->commit_updates>XAPIAN_COMMIT_ENTRIES) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) )
 	{
-		if(verbose>0) i_info("FTS Xapian: Refreshing after %ld ms (vs %ld) and %ld updates (vs %ld) ...", current_time - backend->commit_time, XAPIAN_COMMIT_TIMEOUT*1000, backend->commit_updates, XAPIAN_COMMIT_ENTRIES);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Refreshing after %ld ms (vs %ld) and %ld updates (vs %ld) ...", current_time - backend->commit_time, XAPIAN_COMMIT_TIMEOUT*1000, backend->commit_updates, XAPIAN_COMMIT_ENTRIES);
 		fts_backend_xapian_release(backend,"refreshing", current_time);
 	}
 
@@ -576,12 +501,12 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 {
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *) _backend;
 
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_optimize '%s'",backend->path);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_optimize '%s'",backend->path);
 
 	struct stat sb;
 	if(!( (stat(backend->path, &sb)==0) && S_ISDIR(sb.st_mode)))
 	{
-		if(verbose>0) i_error("FTS Xapian: Index folder inexistent");
+		if(fts_xapian_settings.verbose>0) i_error("FTS Xapian: Index folder inexistent");
 		return -1;
 	}
 
@@ -594,12 +519,12 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 
 		if((dp->d_type == DT_REG) && (strncmp(dp->d_name,"expunge_",8)==0))
 		{
-			if(verbose>0) i_info("Removing %s",s);
+			if(fts_xapian_settings.verbose>0) i_info("Removing %s",s);
 			remove(s);
 		}
 		else if((dp->d_type == DT_DIR) && (strncmp(dp->d_name,"db_",3)==0))
 		{
-			if(verbose>0) i_info("Expunging %s",s);
+			if(fts_xapian_settings.verbose>0) i_info("Expunging (7) %s",s);
 			fts_backend_xapian_do_expunge(s);
 		}
 		i_free(s);
@@ -610,7 +535,7 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 
 static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_rescan");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_rescan");
 
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *) _backend;
 
@@ -630,7 +555,7 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 
 		if((dp->d_type == DT_REG) && (strncmp(dp->d_name,"expunge_",8)==0))
 		{
-			if(verbose>0) i_info("Removing[1] %s",s);
+			if(fts_xapian_settings.verbose>0) i_info("Removing[1] %s",s);
 			remove(s);
 		}
 		else if((dp->d_type == DT_DIR) && (strncmp(dp->d_name,"db_",3)==0))
@@ -642,13 +567,13 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 				s2 = i_strdup_printf("%s/%s",s,dp2->d_name);
 				if(dp2->d_type == DT_REG)
 				{
-					if(verbose>0) i_info("Removing[2] %s",s2);
+					if(fts_xapian_settings.verbose>0) i_info("Removing[2] %s",s2);
 					remove(s2);
 				}
 				i_free(s2);
 			}
 			closedir(d2);
-			if(verbose>0) i_info("Removing dir %s",s);
+			if(fts_xapian_settings.verbose>0) i_info("Removing dir %s",s);
 			remove(s);
 		}
 		i_free(s);
@@ -660,7 +585,7 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 
 static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbox *box, struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_result *result)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup");
 
 	struct xapian_fts_backend *backend = (struct xapian_fts_backend *) _backend;
 
@@ -683,21 +608,21 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 
 	if((flags & FTS_LOOKUP_FLAG_AND_ARGS) != 0)
 	{
-		if(verbose>1) i_info("FTS Xapian: FLAG=AND");
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: FLAG=AND");
 		is_and=true;
 	}
 	else
 	{
-		if(verbose>1) i_info("FTS Xapian: FLAG=OR");
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: FLAG=OR");
 	}
 
-	XQuerySet * qs = new XQuerySet(is_and,false,backend->partial);
+	XQuerySet * qs = new XQuerySet(is_and,false,fts_xapian_settings.partial);
 	fts_backend_xapian_build_qs(qs,args);
 
 	XResultSet * r=fts_backend_xapian_query(dbr,qs);
 
 	long n=r->size;
-	if(verbose>0) { i_info("FTS Xapian: QUery '%s' -> %ld results",qs->get_string().c_str(),n); }
+	if(fts_xapian_settings.verbose>0) { i_info("FTS Xapian: QUery '%s' -> %ld results",qs->get_string().c_str(),n); }
 
 	i_array_init(&(result->definite_uids),r->size);
 
@@ -721,14 +646,14 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 	delete(dbr);
 
 	/* Performance calc */
-	if(verbose>0) i_info("FTS Xapian: %ld results in %ld ms",n,fts_backend_xapian_current_time() - current_time);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: %ld results in %ld ms",n,fts_backend_xapian_current_time() - current_time);
 
 	return 0;
 }
 
 static int fts_backend_xapian_lookup_multi(struct fts_backend *_backend, struct mailbox *const boxes[], struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_multi_result *result)
 {
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup_multi");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup_multi");
 
 	ARRAY(struct fts_result) box_results;
 
